@@ -17,6 +17,7 @@
 #include <utility>  // for pair
 #include <vector>
 
+#include <gio/gio.h>  // for GMemoryMonitor
 #include <glib-object.h>
 #include <glib.h>
 
@@ -27,12 +28,12 @@
 #include <js/GCVector.h>
 #include <js/HashTable.h>  // for DefaultHasher
 #include <js/Promise.h>
+#include <js/Realm.h>
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
 #include <js/UniquePtr.h>
 #include <js/ValueArray.h>
 #include <jsfriendapi.h>  // for ScriptEnvironmentPreparer
-#include <mozilla/Vector.h>
 
 #include "gi/closure.h"
 #include "gjs/context.h"
@@ -82,6 +83,7 @@ class GjsContextPrivate : public JS::JobQueue {
     JobQueueStorage m_job_queue;
     Gjs::PromiseJobDispatcher m_dispatcher;
     Gjs::MainLoop m_main_loop;
+    GjsAutoUnref<GMemoryMonitor> m_memory_monitor;
 
     std::vector<std::pair<DestroyNotify, void*>> m_destroy_notifications;
     std::vector<Gjs::Closure::Ptr> m_async_closures;
@@ -218,6 +220,9 @@ class GjsContextPrivate : public JS::JobQueue {
     [[nodiscard]] static const GjsAtoms& atoms(JSContext* cx) {
         return *(from_cx(cx)->m_atoms);
     }
+    [[nodiscard]] static JSObject* global(JSContext* cx) {
+        return from_cx(cx)->global();
+    }
 
     [[nodiscard]] bool eval(const char* script, size_t script_len,
                             const char* filename, int* exit_status_p,
@@ -276,4 +281,18 @@ class GjsContextPrivate : public JS::JobQueue {
 
 std::string gjs_dumpstack_string();
 
+namespace Gjs {
+class AutoMainRealm : public JSAutoRealm {
+ public:
+    explicit AutoMainRealm(GjsContextPrivate* gjs);
+    explicit AutoMainRealm(JSContext* cx);
+};
+
+class AutoInternalRealm : public JSAutoRealm {
+ public:
+    explicit AutoInternalRealm(GjsContextPrivate* gjs);
+    explicit AutoInternalRealm(JSContext* cx);
+};
+}  // namespace Gjs
+
 #endif  // GJS_CONTEXT_PRIVATE_H_
