@@ -51,8 +51,9 @@ class GjsBaseGlobal {
         // Enable WeakRef without the cleanupSome specification
         // Re-evaluate if cleanupSome is standardized
         // See: https://github.com/tc39/proposal-cleanup-some
-        options.setWeakRefsEnabled(
-            JS::WeakRefSpecifier::EnabledWithoutCleanupSome);
+        options
+            .setWeakRefsEnabled(JS::WeakRefSpecifier::EnabledWithoutCleanupSome)
+            .setChangeArrayByCopyEnabled(true);
 
         JS::RealmBehaviors behaviors;
         JS::RealmOptions compartment_options(options, behaviors);
@@ -132,8 +133,8 @@ class GjsBaseGlobal {
 
         JS::RootedObject native_obj(m_cx);
 
-        if (!Gjs::NativeModuleRegistry::get().load(m_cx, id.get(),
-                                                   &native_obj)) {
+        if (!Gjs::NativeModuleDefineFuncs::get().define(m_cx, id.get(),
+                                                        &native_obj)) {
             gjs_throw(m_cx, "Failed to load native module: %s", id.get());
             return false;
         }
