@@ -97,6 +97,7 @@ var TestIface = `<node>
 <property name="PropReadOnly" type="d" access="read" />
 <property name="PropWriteOnly" type="s" access="write" />
 <property name="PropReadWrite" type="v" access="readwrite" />
+<property name="PropPrePacked" type="a{sv}" access="read" />
 </interface>
 </node>`;
 
@@ -222,6 +223,12 @@ class Test {
         this._propReadWrite = value.deepUnpack();
     }
 
+    get PropPrePacked() {
+        return new GLib.Variant('a{sv}', {
+            member: GLib.Variant.new_string('value'),
+        });
+    }
+
     structArray() {
         return [[128, 123456], [42, 654321]];
     }
@@ -335,6 +342,14 @@ describe('Exported DBus object', function () {
         expect(hello.deepUnpack()).toEqual('world');
     });
 
+    it('can initiate a proxy with promise and call a method with async/await', async function () {
+        const asyncProxy = await ProxyClass.newAsync(Gio.DBus.session,
+            'org.gnome.gjs.Test', '/org/gnome/gjs/Test');
+        expect(asyncProxy).toBeInstanceOf(Gio.DBusProxy);
+        const [{hello}] = await asyncProxy.frobateStuffAsync({});
+        expect(hello.deepUnpack()).toEqual('world');
+    });
+
     it('can call a remote method when not using makeProxyWrapper', function () {
         let info = Gio.DBusNodeInfo.new_for_xml(TestIface);
         let iface = info.interfaces[0];
@@ -775,6 +790,7 @@ describe('Exported DBus object', function () {
         expect(proxy.hasOwnProperty('PropReadWrite')).toBeTruthy();
         expect(proxy.hasOwnProperty('PropReadOnly')).toBeTruthy();
         expect(proxy.hasOwnProperty('PropWriteOnly')).toBeTruthy();
+        expect(proxy.hasOwnProperty('PropPrePacked')).toBeTruthy();
     });
 
     it('reading readonly property works', function () {
@@ -820,6 +836,11 @@ describe('Exported DBus object', function () {
         expect(proxy.PropReadOnly).toBe(PROP_READ_ONLY_INITIAL_VALUE);
     });
 
+    it('Reading a property that prepacks the return value works', function () {
+        expect(proxy.PropPrePacked.member).toBeInstanceOf(GLib.Variant);
+        expect(proxy.PropPrePacked.member.get_type_string()).toEqual('s');
+    });
+
     it('Marking a property as invalidated works', function () {
         let changedProps = {};
         let invalidatedProps = [];
@@ -905,6 +926,20 @@ describe('DBus Proxy wrapper', function () {
         expect(writerFunc).not.toHaveBeenCalled();
     });
 
+    it('can create a proxy from a promise', async function () {
+        const proxyPromise = ProxyClass.newAsync(Gio.DBus.session, 'org.gnome.gjs.Test',
+            '/org/gnome/gjs/Test');
+        await expectAsync(proxyPromise).toBeResolved();
+    });
+
+    it('can create fail a proxy from a promise', async function () {
+        const cancellable = new Gio.Cancellable();
+        cancellable.cancel();
+        const proxyPromise = ProxyClass.newAsync(Gio.DBus.session, 'org.gnome.gjs.Test',
+            '/org/gnome/gjs/Test', cancellable);
+        await expectAsync(proxyPromise).toBeRejected();
+    });
+
     afterAll(function () {
         if (!wasPromise) {
             // Remove stuff added by Gio._promisify, this can be not needed in future
