@@ -7,7 +7,6 @@
 #include <cairo.h>
 
 #include <js/CallArgs.h>
-#include <js/GCPolicyAPI.h>
 #include <js/PropertyDescriptor.h>  // for JSPROP_READONLY
 #include <js/PropertySpec.h>
 #include <js/RootingAPI.h>
@@ -20,12 +19,6 @@
 #include "gjs/macros.h"
 #include "modules/cairo-private.h"
 
-// Avoid static_assert in MSVC builds
-namespace JS {
-template <>
-struct GCPolicy<void*> : public IgnoreGCPolicy<void*> {};
-}
-
 JSObject* CairoGradient::new_proto(JSContext* cx, JSProtoKey) {
     JS::RootedObject parent_proto(cx, CairoPattern::prototype(cx));
     return JS_NewObjectWithGivenProto(cx, nullptr, parent_proto);
